# Copyright (c) 1999-2008 Mark D. Hill and David A. Wood
# Copyright (c) 2009 The Hewlett-Packard Development Company
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met: redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer;
# redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution;
# neither the name of the copyright holders nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from slicc.util import PairContainer, Location

class AST(PairContainer):
    def __init__(self, slicc, pairs=None):
        self.slicc = slicc
        self.location = Location(slicc.current_file, slicc.lexer.lineno)
        self.pairs = {}
        if pairs:
            self.pairs.update(getattr(pairs, "pairs", pairs))

    @property
    def symtab(self):
        return self.slicc.symtab

    @property
    def state_machine(self):
        return self.slicc.symtab.state_machine

    def warning(self, message, *args):
        self.location.warning(message, *args)

    def error(self, message, *args):
        self.location.error(message, *args)

    def embedError(self, message, *args):
        if args:
            message = message % args
        code = self.slicc.codeFormatter()
        code('''
char c;
cerr << "Runtime Error at ${{self.location}}, Ruby Time: "
     << g_eventQueue_ptr->getTime() << ": "
     << $message
     << ", PID: " << getpid() << endl
     << "press return to continue." << endl;
cin.get(c);
abort();
''')

        return code
